<?php

namespace Drupal\recombee\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\Entity\ConfigEntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Recombee Public Scenario' block.
 *
 * @Block(
 *   id = "recombee_public_scenario",
 *   admin_label = @Translation("Recombee Public Scenario"),
 *   category = @Translation("Search")
 * )
 */
class RecombeePublicScenario extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The Recombee configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $recombeeConfig;

  /**
   * The Recombee template entity storage.
   *
   * @var \Drupal\Core\Config\Entity\ConfigEntityStorageInterface
   */
  protected $templateStorage;

  /**
   * Constructs a new RecombeePublicScenario instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\Config $recombee_config
   *   The Recombee configuration.
   * @param \Drupal\Core\Config\Entity\ConfigEntityStorageInterface $template_storage
   *   The Recombee template entity storage.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Config $recombee_config, ConfigEntityStorageInterface $template_storage) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->recombeeConfig = $recombee_config;
    $this->templateStorage = $template_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')->get('recombee.settings'),
      $container->get('entity_type.manager')->getStorage('recombee_template')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'endpoint' => '',
      'scenario' => '',
      'template' => '',
      'page_size' => 5,
      'account_id' => '',
      'public_token' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->configuration;

    // Get all defined templates.
    $template_options = [];
    foreach ($this->templateStorage->loadMultiple() as $template) {
      $template_options[$template->id()] = $template->label();
    }

    // Prepare description links.
    $recombee_link = Link::fromTextAndUrl('Recombee', Url::fromUri('https://admin.recombee.com/'))
      ->toString();
    $settings_link = Link::createFromRoute('Configure default', 'recombee.settings')
      ->toString();
    $templates_link = Link::createFromRoute('Manage templates', 'entity.recombee_template.collection')
      ->toString();

    $form['endpoint'] = [
      '#type' => 'select',
      '#title' => $this->t('Recommendation type'),
      '#description' => $this->t('The @recombee Scenario recommendation type.', [
        '@recombee' => $recombee_link,
      ]),
      '#options' => [
        'items-to-item' => $this->t('Items to Item'),
        'items-to-user' => $this->t('Items to User'),
      ],
      '#default_value' => $config['endpoint'],
      '#required' => TRUE,
    ];
    $form['scenario'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Scenario'),
      '#description' => $this->t('The @recombee Scenario to use when returning results (see "Scenarios" page for a database).', [
        '@recombee' => $recombee_link,
      ]),
      '#default_value' => $config['scenario'],
      '#required' => TRUE,
    ];
    $form['template'] = [
      '#type' => 'select',
      '#title' => $this->t('Template'),
      '#description' => $this->t('The template to use to display the results. @link.', [
        '@link' => $templates_link,
      ]),
      '#options' => $template_options,
      '#default_value' => $config['template'],
      '#required' => TRUE,
    ];
    $form['page_size'] = [
      '#type' => 'number',
      '#title' => $this->t('Page size'),
      '#description' => $this->t('The number of results to return.'),
      '#default_value' => $config['page_size'],
      '#required' => TRUE,
      '#min' => 1,
    ];

    $form['api'] = [
      '#type' => 'details',
      '#title' => $this->t('Override Public API credentials'),
      '#open' => empty($this->recombeeConfig->get('account_id')) || empty($this->recombeeConfig->get('public_token')),
    ];
    $form['api']['account_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Database ID'),
      '#description' => $this->t('The @recombee "API Identifier" value (see "Settings" page for a database). @link.', [
        '@recombee' => $recombee_link,
        '@link' => $settings_link,
      ]),
      '#default_value' => $config['account_id'],
      '#required' => empty($this->recombeeConfig->get('account_id')),
      '#placeholder' => $this->recombeeConfig->get('account_id'),
    ];
    $form['api']['public_token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Public token'),
      '#description' => $this->t('The @recombee "Public token" value (see "Settings" page for a database). @link.', [
        '@recombee' => $recombee_link,
        '@link' => $settings_link,
      ]),
      '#default_value' => $config['public_token'],
      '#required' => empty($this->recombeeConfig->get('public_token')),
      '#placeholder' => $this->recombeeConfig->get('public_token'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['endpoint'] = $form_state->getValue('endpoint');
    $this->configuration['scenario'] = trim($form_state->getValue('scenario'));
    $this->configuration['template'] = $form_state->getValue('template');
    $this->configuration['page_size'] = intval($form_state->getValue('page_size'));
    $this->configuration['account_id'] = trim($form_state->getValue('account_id'));
    $this->configuration['public_token'] = trim($form_state->getValue('public_token'));
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $template = $this->templateStorage->load($this->configuration['template']);
    if (empty($template)) {
      return [
        '#type' => 'markup',
        '#markup' => $this->t('Template "@id" was not found.', [
          '@id' => $this->configuration['template'],
        ]),
      ];
    }
    if (empty($this->configuration['account_id'])) {
      $this->configuration['account_id'] = $this->recombeeConfig->get('account_id');
    }
    if (empty($this->configuration['public_token'])) {
      $this->configuration['public_token'] = $this->recombeeConfig->get('public_token');
    }
    return [
      '#theme' => 'recombee_public_scenario',
      '#endpoint' => $this->configuration['endpoint'],
      '#scenario' => $this->configuration['scenario'],
      '#template' => $this->configuration['template'],
      '#page_size' => $this->configuration['page_size'],
      '#account_id' => $this->configuration['account_id'],
      '#public_token' => $this->configuration['public_token'],
      '#attached' => [
        'drupalSettings' => [
          'recombeeTemplate' => [
            $this->configuration['template'] => $template->getMarkup(),
          ],
        ],
      ],
    ];
  }

}
